/* 
 * This file is part of battwd. See battwd.c for further information.
 *
 * battwd is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * battwd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with battwd.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GLOBAL_H
#define GLOBAL_H

#define VERSION "0.1.1"

#define TRUE   1
#define FALSE  0

#define ERR_READING_STATE   1
#define ERR_CLOSING_STATE   1<<1
#define ERR_EXEC_ACTION     1<<2
#define ERR_CHILD_PROC      1<<3
#define ERR_CHILD_SID       1<<4
#define ERR_CHDIR           1<<5
#define INT_BY_SIGNAL       1<<6   

#define BAT_PRESENT         0
#define BAT_CAPACITY        1
#define BAT_CHARGING        2
#define BAT_RATE            3
#define BAT_REMAINING       4
#define BAT_VOLTAGE         5
#define BAT_ALARM           6
#define AC_ADAPTER          7

/* Default AC power on values */
#define ON_LINE_0           "on-line"
#define ON_LINE_1           "1"

/* Field delimiter within monitors files  */
#define DELIMITER_0         ':'
#define DELIMITER_1         '='

#define ARGV_SIZE           3    /* Max. arguments for each control script */
#define FILE_SIZE           16   /* Max. number of lines by file */
#define BUF_SIZE            16   /* Max. size for strings buffers */
#define MAX_ACTIONS         8    /* Max. number of actions by battery */
#define MAX_FILES           3    /* Max. files by battery */

#define LENGTH(x)       (sizeof(x)/sizeof(*x))

typedef struct {
  char *technology;
  int desing_capacity;
  int last_full_capacity;
  int desing_voltage;
  int serial_number;
} battery_model;

/**
 * bat_present   - If battery is present or not
 * bat_capacity  - Battery health
 * bat_charging  - Charging state
 * bat_rate      - Present rate (mW)
 * bat_remaining - Remaining capacity (mWh)
 * bat_voltage   - Present voltage (mV)
 * bat_alarm     - Alarm capacity
 * ac_adapter    - If AC adaptes is present or not
 */
typedef struct {
  char *bat_present;
  char *bat_capacity;
  char *bat_charging;
  int bat_rate;
  int bat_remaining;
  int bat_voltage;
  int bat_alarm;
  char *ac_adapter;
} power_status; 

/**
 * nline - Line number on measure file
 * param - Represent a measure parameter of power_status struct
 */
typedef struct {
  const int nline;
  const int param;
} assoc;

/**
 * path      - Path of the measure files on proc file system
 * offset    - Offset between each line and his power_status structure field
 */
typedef struct {
  const char *path;
  const int delimiter;
  const int lines;
  const assoc alist[FILE_SIZE];
} mon_file;

/**
 * level - Execute an action when the battery charge is below the value of
 * the "level" multiplied by the "charge alarm" value.
 * path  - Path of the script or command that must be executed
 * argv  - Script or command's arguments
 */
typedef struct {
  const int level;
  const char *path;
  const char *const argv[ARGV_SIZE];
} action;

typedef struct {
  const char *id;
  battery_model *model;
  power_status *status;
  const int num_files;
  const mon_file files[MAX_FILES];
  const int num_actions;
  const action actions[MAX_ACTIONS];
  volatile int *action_flags;
} battery;

/* Public variables */
volatile int daemon_flag;

/* Public functions */
extern int run_daemon();

#endif
